@@ -28,6 +28,8 @@ class Net::HTTP
 end
 
 class MiniPortile
+  DEFAULT_TIMEOUT = 10
+
   attr_reader :name, :version, :original_host
   attr_writer :configure_options
   attr_accessor :host, :files, :patch_files, :target, :logger, :source_directory
@@ -46,7 +48,7 @@ class MiniPortile
     RbConfig::CONFIG['target_os'] =~ /mswin/
   end
 
-  def initialize(name, version)
+  def initialize(name, version, **kwargs)
     @name = name
     @version = version
     @target = 'ports'
@@ -57,6 +59,11 @@ class MiniPortile
     @source_directory = nil
 
     @original_host = @host = detect_host
+
+    @gcc_command = kwargs[:gcc_command]
+    @make_command = kwargs[:make_command]
+    @open_timeout = kwargs[:open_timeout] || DEFAULT_TIMEOUT
+    @read_timeout = kwargs[:read_timeout] || DEFAULT_TIMEOUT
   end
 
   def source_directory=(path)
@@ -222,6 +229,14 @@ class MiniPortile
     File.expand_path(port_path)
   end
 
+  def gcc_cmd
+    (ENV["CC"] || @gcc_command || RbConfig::CONFIG["CC"] || "gcc").dup
+  end
+
+  def make_cmd
+    (ENV["MAKE"] || @make_command || ENV["make"] || "make").dup
+  end
+
 private
 
   def tmp_path
@@ -347,6 +362,8 @@ private
         'z'
       when '.bz2', '.tbz2'
         'j'
+      when '.xz'
+        'J'
       when '.Z'
         'Z'
       else
@@ -501,7 +518,9 @@ private
             # Content-Length is unavailable because Transfer-Encoding is chunked
             message "\rDownloading %s " % [filename]
           end
-        }
+        },
+        :open_timeout => @open_timeout,
+        :read_timeout => @read_timeout,
       }
       proxy_uri = URI.parse(url).scheme.downcase == 'https' ?
                   ENV["https_proxy"] :
@@ -526,7 +545,7 @@ private
         return download_file(redirect.url, full_path, count-1)
       rescue => e
         count = count - 1
-        puts "#{count} retrie(s) left for #{filename}"
+        puts "#{count} retrie(s) left for #{filename} (#{e.message})"
         if count > 0
           sleep 1
           return download_file_http(url, full_path, count)
@@ -553,7 +572,9 @@ private
         :progress_proc => lambda{|bytes|
           new_progress = (bytes * 100) / total
           message "\rDownloading %s (%3d%%) " % [filename, new_progress]
-        }
+        },
+        :open_timeout => @open_timeout,
+        :read_timeout => @read_timeout,
       }
       if ENV["ftp_proxy"]
         _, userinfo, _p_host, _p_port = URI.split(ENV['ftp_proxy'])
@@ -583,14 +604,4 @@ private
     FileUtils.mkdir_p File.dirname(full_path)
     FileUtils.mv temp_file.path, full_path, :force => true
   end
-
-  def gcc_cmd
-    cc = ENV["CC"] || RbConfig::CONFIG["CC"] || "gcc"
-    return cc.dup
-  end
-
-  def make_cmd
-    m = ENV['MAKE'] || ENV['make'] || 'make'
-    return m.dup
-  end
 end
